<?php

namespace Dj3c1t\SourceMLBundle\Form\Type;

use Symfony\Component\DependencyInjection\ContainerInterface as Container;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormError;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Doctrine\Bundle\DoctrineBundle\Registry as Doctrine;
use Doctrine\ORM\EntityRepository;

use Dj3c1t\SourceMLBundle\Entity\SourceInfo;
use Dj3c1t\SourceMLBundle\Entity\SourceAuthor;

class SourceType extends AbstractType {

    private $em;
    private $sm;
    private $sourceRepo;
    private $licenceRepo;
    private $source;
    private $isNew;

    private $container;
    private $user;

    private $options;

    public function __construct(Container $container) {
        $this->container = $container;
        $this->em = $this->container->get('doctrine')->getManager();
        $this->sm = $this->container->get('sourceml.source_manager');
        $this->sourceRepo = $this->em->getRepository("Dj3c1tSourceMLBundle:Source");
        $this->licenceRepo = $this->em->getRepository("Dj3c1tSourceMLBundle:Licence");
        if(!($this->user = $this->container->get('security.token_storage')->getToken()->getUser())) {
            throw new \Exception("user must be logged in to access source form");
        }
    }

    public function buildForm(FormBuilderInterface $builder, array $options) {
        $this->options = $options;
        $this->source = $builder->getData();
        $this->isNew = ($this->source->getId()) === null;
        if($this->isNew || $this->options["isReference"]) {
            $option = array(
                'class' => 'Dj3c1tSourceMLBundle:Author',
                'label' => $this->options["isReference"] ? 'Imported by' : 'Author',
                'mapped' => false,
                'query_builder' => function (EntityRepository $er) {
                    return $er->createQueryBuilder('a')
                        ->join('a.user', 'u')
                        ->where('u.id = :user_id')
                        ->setParameter('user_id', $this->user->getId());
                },
            );
            if($this->options["isReference"]) {
                $option["data"] = $this->source->getImportedBy();
            }
            $builder->add(
                'author',
                'entity',
                $option
            );
        }
        if($this->options["isReference"]) {
            $builder->add(
                'referenceUrl',
                'text',
                array(
                    'label' => 'XML URL'
                )
            );
        }
        else {
            if(($description = $this->source->getInfo("description")) === null) {
                $description = "";
            }
            $builder->add(
                'title',
                'text',
                array(
                    'label' => 'Title'
                )
            );
            $builder->add(
                'description',
                'textarea',
                array(
                    'label' => 'Description',
                    'data' => $description,
                    'mapped' => false,
                )
            );

            if(
                $this->licenceRepo->createQueryBuilder('l')
                    ->select('COUNT(l)')
                    ->getQuery()->getSingleScalarResult()
            ) {
                $builder->add(
                    'licence',
                    'entity',
                    array(
                        'class' => 'Dj3c1tSourceMLBundle:Licence',
                        'label' => 'Licence',
                    )
                );
            }
        }
        $compositionType = null;
        $compositionLabel = "";
        switch($this->source->getSourceType()->getName()) {
            case "track":
                $compositionType = $this->sm->getSourceType("album");
                $compositionLabel = "In album";
                break;
            case "source":
                $compositionType = $this->sm->getSourceType("track");
                $compositionLabel = "In track";
                break;
        }
        if(isset($compositionType)) {
            $composition = null;
            if(!$this->isNew) {
                foreach($this->source->getCompositions() as $sourceComposition) {
                    $composition = $sourceComposition->getComposition();
                    break;
                }
            }
            if(
                    $this->isNew
                ||  !isset($composition)
                ||  $this->sm->userCan("contribute", $composition)
            ) {
                if(
                    $compositions = $this->sourceRepo->getSourceQuery(
                        array(
                            "user" => $this->user,
                            "sourceType" => $compositionType->getName(),
                            "isReference" => false,
                        )
                    )->getResult()
                ) {
                    $builderParams = array(
                        'class' => 'Dj3c1tSourceMLBundle:Source',
                        'label' => $compositionLabel,
                        'mapped' => false,
                        'choices' => $compositions,
                        'required' => false,
                    );
                    if(isset($composition)) {
                        $builderParams["data"] = $composition;
                    }
                    $builder->add(
                        'composition',
                        'entity',
                        $builderParams
                    );
                }
            }
        }
        $builder->addEventListener(
            FormEvents::SUBMIT,
            array($this, 'onSubmit')
        );
        $builder->addEventListener(
            FormEvents::PRE_SUBMIT,
            array($this, 'onPreSubmit')
        );
    }

    public function getName() {
        return 'sourceml_source';
    }

    public function configureOptions(OptionsResolver $resolver) {
        $resolver->setDefaults(
            array(
                'data_class' => 'Dj3c1t\SourceMLBundle\Entity\Source',
                'translation_domain' => 'sourceml',
                'isReference' => false,
            )
        );
    }

    public function onPreSubmit(FormEvent $event) {
        if(!$this->isNew) {
            if(!$this->sm->userCan("edit", $this->source)) {
                throw new \Exception("your are not allowed to edit this source");
            }
        }
    }

    public function onSubmit(FormEvent $event) {
        $form = $event->getForm();

        // SourceInfo : description

        if(!$this->options["isReference"]) {
            $sourceInfo = null;
            foreach($this->source->getInfos() as $info) {
                if($info->getInfoKey() == "description") {
                    $sourceInfo = $info;
                }
            }
            if(!isset($sourceInfo)) {
                $sourceInfo = new SourceInfo();
                $sourceInfo->setSource($this->source);
                $sourceInfo->setInfoKey("description");
                $this->em->persist($sourceInfo);
                $this->source->addInfo($sourceInfo);
            }
            $sourceInfo->setInfoValue($form->get('description')->getData());
        }

        // SourceAuthor

        if($this->isNew || $this->options["isReference"]) {
            if($authors = $this->source->getAuthors()) {
                foreach($authors as $sourceAuthor) {
                    $this->source->removeAuthor($sourceAuthor);
                    $this->em->remove($sourceAuthor);
                }
            }
            if(!($author = $form->get('author')->getData())) {
                throw new \Exception("source author is required");
            }
            $sourceAuthor = new SourceAuthor();
            $sourceAuthor->setAuthor($author);
            $sourceAuthor->setSource($this->source);
            $sourceAuthor->setAuthorRole($this->sm->getAuthorRole("admin"));
            $sourceAuthor->setIsValid(true);
            $this->em->persist($sourceAuthor);
            $this->source->addAuthor($sourceAuthor);
        }

        // composition

        if($form->has('composition')) {
            $this->sm->setComposition($this->source, $form->get('composition')->getData());
        }

    }

}
